{******************************************}
{                                          }
{             FastReport v3.0              }
{  IB Objects (Native) enduser components  }
{             IB Objects 4.5               }
{  Version-Date: 2005-06-14                }
{					                                 }
{ Created by: Andreas Hesse                }
{ E-mail: delphi@ah3000.de                 }
{                                          }
{******************************************}

unit frxIB_Editor;

interface

implementation

uses
  SysUtils,
	Classes,
  Controls,
  IB_Components,
  IBF_Query,
  frxClass,
	frxDsgnIntf,
  frxIB_Components,
  frxEditIB_QueryParams,
  frxIBOSet,
  frxRes,
  frxEditStrings;

type

  TfrxIB_StringReadOnlyProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
  end;

  (** @see TfrxStringsProperty *)
  TfrxIB_StringsProperty = class(TfrxClassProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    function Edit: Boolean; override;
    function GetValue: string; override;
  end;

  TfrxIB_SqlProperty = class(TfrxClassProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    function Edit: Boolean; override;
    function GetValue: string; override;
  end;

  TfrxIB_ParametersProperty = class(TfrxClassProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    function Edit: Boolean; override;
  end;

  (** @see frxCustomDBEditor *)
  TfrxIB_DataSetProperty = class(TfrxComponentProperty)
  public
    function GetValue: String; override;
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
    procedure SetValue(const Value: String); override;
  end;

  TfrxIB_DatabaseProperty = class(TfrxComponentProperty)
  public
    function GetValue: String; override;
  end;


  TfrxIB_QueryEditor = class(TfrxComponentEditor)
  public
    function Edit: Boolean; override;
    function HasEditor: Boolean; override;
  end;


function GetFirstLine(const AStrings: TStrings): string;
begin
  Result := '';
  if Assigned(AStrings) then begin
    if AStrings.Count = 1 then Result := AStrings[0]
    else if AStrings.Count > 1 then Result := AStrings[0] + ' ...';
  end;
end;

{ TfrxIB_ReadOnly }

function TfrxIB_StringReadOnlyProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paReadOnly];
end;

{ TfrxIB_StringsProperty }

function TfrxIB_StringsProperty.Edit: Boolean;
var
  l: TStrings;
begin
  with TfrxStringsEditorForm.Create(Designer) do
  begin
    l := TStrings(GetOrdValue);
    Memo.Lines.Assign(l);
    Result := ShowModal = mrOk;
    if Result then
      l.Assign(Memo.Lines);
    Free;
  end;
end;

function TfrxIB_StringsProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TfrxIB_StringsProperty.GetValue: string;
begin
  Result := GetFirstLine(TStrings(GetOrdValue));
end;

{ TfrxIB_SqlProperty }

function TfrxIB_SqlProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TfrxIB_SqlProperty.Edit: Boolean;
var
  l: TStrings;
begin
  with TfrxStringsEditorForm.Create(Designer) do
  begin
    l := TStrings(GetOrdValue);
    Memo.Lines.Assign(l);
    Result := ShowModal = mrOk;
    if Result then
    begin
      l.Assign(Memo.Lines);
      TfrxIB_Query(frComponent).UpdateDataTree;
    end;
    Free;
  end;
end;

function TfrxIB_SqlProperty.GetValue: string;
begin
  Result := GetFirstLine(TStrings(GetOrdValue));
end;

{ TfrxIB_ParametersProperty }

function TfrxIB_ParametersProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

(**
 * Edit Parameters.
 * It is possible to define parameters even if the query does not contain any.
 * So you can change the query in code.
 *)
function TfrxIB_ParametersProperty.Edit: Boolean;
begin
  with TfrxIB_ParamsEditorForm.Create(Designer) do
  begin
    Params := TfrxIB_Query(Component).Params;
    Result := ShowModal = mrOk;
    Free;
  end;
end;

{ TfrxDataSetProperty }

function TfrxIB_DataSetProperty.GetAttributes: TfrxPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList];
end;

function TfrxIB_DataSetProperty.GetValue: String;
var
  ds: TfrxDataSet;
begin
  ds := TfrxDataSet(GetOrdValue);
  if ds <> nil then
    Result := frComponent.Report.GetAlias(ds) else
    Result := frxResources.Get('prNotAssigned');
end;

procedure TfrxIB_DataSetProperty.GetValues;
var
  i: Integer;
begin
  frComponent.Report.GetDataSetList(Values, True);
  if Component is TfrxDataSet then
  begin
    i := Values.IndexOf(TfrxDataSet(Component).UserName);
    if i <> -1 then
      Values.Delete(i);
  end;
end;

procedure TfrxIB_DataSetProperty.SetValue(const Value: String);
var
  ds: TfrxDataSet;
begin
  if Value = '' then
    SetOrdValue(0)
  else
  begin
    ds := frComponent.Report.GetDataSet(Value);
    if ds <> nil then
      SetOrdValue(Integer(ds)) else
      raise Exception.Create(frxResources.Get('prInvProp'));
  end;
end;


{ TfrxIB_DatabaseProperty }

function TfrxIB_DatabaseProperty.GetValue: String;
var
  db: TfrxIB_Database;
begin
  db := TfrxIB_Database(GetOrdValue);
  if db = nil then
  begin
    if Assigned(GlobalIB_Components) and Assigned(GlobalIB_Components.DefaultConnection) then
      Result := frxResources.Get('ibDefault')
    else
      Result := frxResources.Get('prNotAssigned');
  end
  else
    Result := inherited GetValue;
end;


{ TfrxIB_QueryEditor }

(** Show IB Objects Dialog on Double Click *)
function TfrxIB_QueryEditor.Edit: Boolean;
var
  Query: TIB_Query;
  EditForm: TfrmQuery;
  FrxQuery: TfrxIB_Query;
begin
  FrxQuery := (Self.Component as TfrxIB_Query);
	Query := FrxQuery.Query;
  EditForm := TfrmQuery.Create( Designer );
  try
    EditForm.Caption := FrxQuery.Name + ' ('+ FrxQuery.UserName + ') - ' + EditForm.Caption;
    EditForm.pnClose.Visible := true;
    EditForm.StatusBar.Visible := true;
    EditForm.dsQuerySource.Dataset := Query;
    if Assigned( Query.MasterSource ) then begin
      EditForm.dsMasterSource.Dataset := Query.MasterSource.Dataset;
    end;
    EditForm.BaseConnection := Query.IB_Connection;
    EditForm.BaseTransaction := Query.IB_Transaction;
    EditForm.btOk.Default := false;
    EditForm.BaseKey := 'Software\FR3\IBO\Editors\Dataset';
    Result := EditForm.ShowModal = mrOK;
  finally
    EditForm.Free;
  end;
  if Result and not Query.Prepared then
    FrxQuery.UpdateDataTree;
end;

function TfrxIB_QueryEditor.HasEditor: Boolean;
begin
  Result := True;
end;

initialization
  frxComponentEditors.Register(TfrxIB_Query, TfrxIB_QueryEditor);

  frxPropertyEditors.Register(TypeInfo(string), TfrxIB_Query, 'Version', TfrxIB_StringReadOnlyProperty);
  frxPropertyEditors.Register(TypeInfo(TStrings), TfrxIB_Query, 'SQL', TfrxIB_SqlProperty);
  frxPropertyEditors.Register(TypeInfo(TfrxIB_Params), TfrxIB_Query, 'Params', TfrxIB_ParametersProperty);
  frxPropertyEditors.Register(TypeInfo(TfrxIB_Database), TfrxIB_Query, 'Database', TfrxIB_DatabaseProperty);

  frxPropertyEditors.Register(TypeInfo(TfrxIBODataset), TfrxIB_Query, 'Master', TfrxIB_DataSetProperty);
  frxPropertyEditors.Register(TypeInfo(TStrings), TfrxIB_Query, 'MasterLinks', TfrxIB_StringsProperty);

  (** @see frxCustomDBEditor *)
  frxHideProperties(TfrxIB_Query, 'DataSet;DataSource;Enabled;OpenDataSource;Tag');

end.
